@@ -1,5 +1,5 @@
 /*
- * $Id: login.c,v 1.5 1997/06/24 14:18:56 morgan Exp $
+ * $Id: login.c,v 1.5 1997/06/24 14:18:56 morgan Exp morgan $
  *
  * $Log: login.c,v $
  * Revision 1.5  1997/06/24 14:18:56  morgan
@@ -30,17 +30,17 @@
  *
  * However, this file contains no code from the above software.
  *
- * Copyright (c) 1996,1997 Andrew G. Morgan <morgan@parc.power.net>
+ * Copyright (c) 1996,1997 Andrew G. Morgan <morgan@linux.kernel.org>
  */
 
 static const char rcsid[] =
-"$Id: login.c,v 1.5 1997/06/24 14:18:56 morgan Exp $\n"
-" - Login application. <morgan@parc.power.net>"
+"$Id: login.c,v 1.5 1997/06/24 14:18:56 morgan Exp morgan $\n"
+" - Login application. <morgan@linux.kernel.org>"
 ;
 
+#define _BSD_SOURCE
 #include <ctype.h>
 #include <fcntl.h>
-#define __USE_BSD
 #include <grp.h>
 #include <paths.h>
 #include <pwd.h>
@@ -72,13 +72,13 @@ static const char rcsid[] =
 #define LOGIN_ABORT_TEXT          "\a..Login canceled!\n"
 
 #define MAX_LOGIN                 3  /* largest tolerated delay */
-#define SLEEP_AFTER_MAX_LOGIN    10  /* failed login => delay before retry */
+#define SLEEP_AFTER_MAX_LOGIN     5  /* failed login => delay before retry */
 
-#define GOODBYE_MESSAGE           "Good-bye!\n"  /* make "" for no message */
+#define GOODBYE_MESSAGE           ""  /* make "" for no message */
 #define GOODBYE_DELAY             1  /* time to display good-bye */
 
 #define SERIOUS_ABORT_DELAY       3600                    /* yes, an hour! */
-#define STANDARD_DELAY            15     /* standard failures lead to this */
+#define STANDARD_DELAY            5      /* standard failures lead to this */
 
 #define SLEEP_TO_KILL_CHILDREN    3  /* seconds to wait after SIGTERM before
 					SIGKILL */
@@ -219,7 +219,7 @@ static void parse_args(int argc, const char **argv)
 
 		    hn = 2+*argv;
 		    if (hn[0] || (--argc > 0 && (hn = *++argv))) {
-			login_remote_host = xstrdup(hn);
+			login_remote_host = x_strdup(hn);
 			hn = NULL;
 			if (login_remote_host != NULL)
 			    break;
@@ -353,7 +353,7 @@ static int login_authenticate_user(void)
 	/* have to pause on failure. At least this long (doubles..) */
 	retval = pam_fail_delay(pamh, delay);
 	if (retval != PAM_SUCCESS) {
-	    D(("Error setting delay; %s", pam_strerror(retval)));
+	    D(("Error setting delay; %s", pam_strerror(pamh,retval)));
 	    return retval;
 	}
 #endif /* HAVE_PAM_FAIL_DELAY */
@@ -367,14 +367,14 @@ static int login_authenticate_user(void)
 	    retval = pam_authenticate(pamh, 0);
 	}
 
-	D(("authentication => %s", pam_strerror(retval)));
+	D(("authentication => %s", pam_strerror(pamh,retval)));
 	if (retval == PAM_SUCCESS) {
 
 	    /* the user was authenticated - can they log in? */
 	    retval = pam_acct_mgmt(pamh, 0);
 
-	    D(("account => %s", pam_strerror(retval)));
-	    if (retval == PAM_SUCCESS || retval == PAM_AUTHTOKEN_REQD) {
+	    D(("account => %s", pam_strerror(pamh,retval)));
+	    if (retval == PAM_SUCCESS || retval == PAM_NEW_AUTHTOK_REQD) {
 		return retval;
 	    }
 
@@ -403,7 +403,7 @@ static int login_authenticate_user(void)
 	case PAM_AUTHINFO_UNAVAIL:
 	case PAM_CONV_ERR:
 	case PAM_SERVICE_ERR:
-	    D(("system failed; %s", pam_strerror(retval)));
+	    D(("system failed; %s", pam_strerror(pamh,retval)));
 	    return retval;
 	default:
 	    fprintf(stderr, LOGIN_ATTEMPT_FAILED);
@@ -417,7 +417,7 @@ static int login_authenticate_user(void)
 	}
 
 	if (retval != PAM_SUCCESS) {
-	    D(("Internal failure; %s",pam_strerror(retval)));
+	    D(("Internal failure; %s",pam_strerror(pamh,retval)));
 	    return retval;
 	}
     }
@@ -482,7 +482,7 @@ static void login_invoke_shell(const char *shell, uid_t uid)
     /* now copy environment */
 
     D(("get the environment"));
-    shell_env = pam_misc_copy_env(pamh);
+    shell_env = pam_getenvlist(pamh);
     if (shell_env == NULL) {
 	exit_program(1, STANDARD_DELAY, PAM_ABORT
 		     , "environment corrupt; sorry..");
@@ -555,9 +555,10 @@ static void login_invoke_shell(const char *shell, uid_t uid)
 void main(int argc, const char **argv)
 {
     static const char *shell=NULL;
-    int retval=LOGIN_FALSE, status, child;
+    int retval=LOGIN_FALSE, status;
+    pid_t child;
     uid_t uid= (uid_t) -1;
-
+    
     /*
      * Parse the arguments to login. There are static variables
      * above that indicate the intentions of the invoking process
@@ -594,7 +595,7 @@ void main(int argc, const char **argv)
     if (retval != PAM_SUCCESS) {
 	exit_program(1, 60, retval
 		     , "Error initializing;\n\t%s\n"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh,retval));
     }
     user = NULL;                   /* reset to avoid possible confusion */
 
@@ -605,7 +606,7 @@ void main(int argc, const char **argv)
     retval = make_environment(pamh, (login_flags & LOGIN_KEEP_ENV));
 
     if (retval == PAM_SUCCESS) {
-	D(("return = %s", pam_strerror(retval)));
+	D(("return = %s", pam_strerror(pamh,retval)));
 	D(("login prompt: %s", login_prompt));
 	retval = pam_set_item( pamh, PAM_USER_PROMPT
 			     , (const void *) login_prompt );
@@ -621,7 +622,7 @@ void main(int argc, const char **argv)
 
     if (retval != PAM_SUCCESS) {
 	exit_program(1, 60, retval, "Internal failure;\n\t%s\n"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh,retval));
     }
 
     /*
@@ -637,7 +638,7 @@ void main(int argc, const char **argv)
     retval = login_authenticate_user();
     switch (retval) {
     case PAM_SUCCESS:
-    case PAM_AUTHTOKEN_REQD:    /* user is user, just need a new password */
+    case PAM_NEW_AUTHTOK_REQD:  /* user is user, just need a new password */
 	break;
     case PAM_MAXTRIES:
 	exit_program(1, SLEEP_AFTER_MAX_LOGIN, retval
@@ -646,7 +647,7 @@ void main(int argc, const char **argv)
     case PAM_SERVICE_ERR:
 	exit_program(1, SLEEP_AFTER_MAX_LOGIN, retval, "Login failed\n");
     default:
-	D(("Login failed; %s", pam_strerror(retval)));
+	D(("Login failed; %s", pam_strerror(pamh,retval)));
 	exit_program(1, STANDARD_DELAY, retval, "Login failed\n");
     }
 
@@ -654,7 +655,7 @@ void main(int argc, const char **argv)
      * Do we need to prompt the user for a new password?
      */
 
-    if (retval == PAM_AUTHTOKEN_REQD) {
+    if (retval == PAM_NEW_AUTHTOK_REQD) {
 	set_timeout(LOGIN_TRUE);
 	retval = pam_chauthtok(pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
 
@@ -662,7 +663,7 @@ void main(int argc, const char **argv)
 	switch (retval) {
 	case PAM_AUTHTOK_LOCK_BUSY:
 	case PAM_TRY_AGAIN:
-	    D(("chauthtok: %s", pam_strerror(retval)));
+	    D(("chauthtok: %s", pam_strerror(pamh,retval)));
 	    retval = PAM_SUCCESS;
 	    fprintf(stderr
 		    , "login: please update your authentication token(s)\n");
@@ -675,7 +676,7 @@ void main(int argc, const char **argv)
 
     if (retval != PAM_SUCCESS) {
 	exit_program(1, STANDARD_DELAY, retval, "Login failure;\n\t%s\n"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh,retval));
     }
 
     set_timeout(LOGIN_FALSE);           /* from here we don't need timeouts */
@@ -689,7 +690,7 @@ void main(int argc, const char **argv)
     if (retval != PAM_SUCCESS) {
 	exit_program(1, STANDARD_DELAY, retval
 		     , "Error opening session;\n\t%s\n"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh,retval));
     }
 
     /*
@@ -699,7 +700,7 @@ void main(int argc, const char **argv)
     D(("establish user credentials"));
     retval = set_user_credentials(pamh, LOGIN_TRUE, &user, &uid, &shell);
     if (retval != PAM_SUCCESS) {
-	D(("failure; %s", pam_strerror(retval)));
+	D(("failure; %s", pam_strerror(pamh,retval)));
 	(void) pam_close_session(pamh, retval);
 	exit_program(1, STANDARD_DELAY, retval, "Credential setting failure");
     }
@@ -737,6 +738,7 @@ void main(int argc, const char **argv)
      * Process is parent here... wait for the child to exit
      */
 
+    prepare_for_job_control(child, 0);
     status = wait_for_child(child);
     if (status != 0) {
 	D(("shell returned %d", status));
@@ -745,10 +747,10 @@ void main(int argc, const char **argv)
     utmp_close_session(pamh, child);
 
     /* Delete the user's credentials. */
-    retval = pam_setcred(pamh, PAM_CRED_DELETE);
+    retval = pam_setcred(pamh, PAM_DELETE_CRED);
     if (retval != PAM_SUCCESS) {
 	fprintf(stderr, "WARNING: could not delete credentials\n\t%s\n"
-		, pam_strerror(retval));
+		, pam_strerror(pamh,retval));
     }
 
     /* close down */
